<?php

/**
 * Acclaro translation plugin controller.
 */
class TMGMTAcclaroTranslatorPluginController extends TMGMTDefaultTranslatorPluginController {

  /**
   * Translation service URL.
   */
  const PRODUCTION_URL = 'https://my.acclaro.com';

  /**
   * Translation sandbox service URL.
   */
  const SANDBOX_URL = 'https://apisandbox.acclaro.com';

  /**
   * Translation service API version.
   *
   * @var string
   */
  const API_VERSION = 'api2';

  /**
   * List of supported languages by Acclaro.
   *
   * @var string[]
   */
  protected $supportedRemoteLanguages = [];

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::isAvailable().
   */
  public function isAvailable(TMGMTTranslator $translator) {
    return (bool) $translator->getSetting('token');
  }

  /**
   * {@inheritdoc}
   */
  public function defaultSettings() {
    $defaults = parent::defaultSettings();
    $defaults['export_format'] = 'xlf';
    // Enable CDATA for content encoding in File translator.
    $defaults['xliff_cdata'] = TRUE;
    $defaults['xliff_processing'] = FALSE;
    return $defaults;
  }

  /**
   * {@inheritdoc}
   */
  public function requestTranslation(TMGMTJob $job) {
    $translator = $job->getTranslator();
    $source_language = $this->mapToRemoteLanguage($translator, $job->source_language);
    $target_language = $this->mapToRemoteLanguage($translator, $job->target_language);
    $xliff = tmgmt_file_format_controller('xlf');
    $job_id = $job->tjid;

    // Create a new order and add its callback route.
    $name = $job->getSetting('name') ? $job->getSetting('name') : $job->label() . ' (' . $job_id . ')';

    try {
      $order = $this->createOrder($translator, $name, $job->getSetting('comment'), $job->getSetting('duedate'));
      $order_id = $order['orderid'];
      $job->addMessage('Order (@order_id) has been created.', ['@order_id' => $order_id]);
      $this->requestOrderCallback($translator, $order_id, $job_id);

      /** @var \TMGMTJobItem $job_item */
      foreach ($job->getItems() as $job_item) {
        $job_item_id = $job_item->tjiid;
        // Export content as XLIFF.
        $xliff_content = $xliff->export($job, ['tjiid' => ['value' => $job_item_id]]);
        // Build a file name.
        $file_name = "JobID_{$job_id}_JobItemID_{$job_item_id}_{$job->source_language}_{$job->target_language}.xlf";

        // Upload a file.
        $file_status = $this->sendSourceFile($translator, $source_language, $target_language, $order_id, $job_item_id, $file_name, $xliff_content);
        $job_item->active();
        // Set a callback route in order to get updates on file status changes.
        $this->requestFileCallback($translator, $order_id, $file_status['fileid'], $job_item_id);
        // Add a remote reference (file ID) to the job item.
        $job_item->addRemoteMapping(NULL, $file_status['fileid'], ['remote_identifier_2' => $order_id]);
      }

      // Submit the order for translation.
      $this->submitOrder($translator, $order_id);
    }
    catch (TMGMTException $e) {
      $job->rejected('Job has been rejected with following error: @error', ['@error' => $e->getMessage()], 'error');
    }

    if (!$job->isRejected()) {
      $job->submitted('Job has been successfully submitted for translation.');
    }
  }

  /**
   * Executes a request against Acclaro API.
   *
   * @param TMGMTTranslator $translator
   *   The translator.
   * @param string $path
   *   Resource path.
   * @param string|array $parameters
   *   (optional) Parameters to send to Acclaro service.
   * @param string $method
   *   (optional) HTTP method (GET, POST...). Defaults to GET.
   * @param array $headers
   *   (optional) A list of request headers to set.
   *
   * @return array
   *   Response array from Acclaro.
   *
   * @throws TMGMTException
   */
  protected function doRequest(TMGMTTranslator $translator, $path, $parameters = [], $method = 'GET', array $headers = []) {
    if ($translator->getSetting('use_sandbox')) {
      $url = self::SANDBOX_URL . '/' . self::API_VERSION . '/' . $path;
    }
    else {
      $url = self::PRODUCTION_URL . '/' . self::API_VERSION . '/' . $path;
    }

    // Add the authorization token.
    $options['headers']['Authorization'] = 'Bearer ' . $translator->getSetting('token');
    $options['headers'] = array_merge($options['headers'], $headers);
    $options['method'] = $method;

    if ($method == 'GET') {
      $url = url($url, ['query' => $parameters]);
    }
    else {
      $options['data'] = $parameters;
    }

    // Make a request.
    $response = drupal_http_request($url, $options);

    if ($response->code != 200) {
      throw new TMGMTException('Unable to connect to the Acclaro service due to following error: @error at @url',
        ['@error' => $response->error, '@url' => $url]);
    }

    $received_data = drupal_json_decode($response->data);
    // In case JSON decoding fails, return the plain body.
    if (!$received_data) {
      return $response->data;
    }

    if (!$received_data['success']) {
      throw new TMGMTException('Acclaro service returned validation error: #%code %error',
        [
          '%code' => $received_data['errorCode'],
          '%error' => $received_data['errorMessage'],
        ]);
    }

    return isset($received_data['data']) ? $received_data['data'] : $received_data;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedTargetLanguages(TMGMTTranslator $translator, $source_language) {
    $target_languages = [];
    $language_pairs = $this->getLanguagePairs($translator, $translator->mapToRemoteLanguage($source_language));

    // Parse languages.
    foreach ($language_pairs as $language_pair) {
      $target_language = $translator->mapToLocalLanguage($language_pair['target']['code']);
      $target_languages[$target_language] = $target_language;
    }

    return $target_languages;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedRemoteLanguages(TMGMTTranslator $translator) {
    try {
      $supported_languages = $this->doRequest($translator, 'GetLanguages');
      // Parse languages.
      foreach ($supported_languages as $language) {
        $this->supportedRemoteLanguages[$language['code']] = $language['code'];
      }
      return $this->supportedRemoteLanguages;
    }
    catch (\Exception $e) {
      return [];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedLanguagePairs(TMGMTTranslator $translator) {
    $language_pairs = [];

    try {
      $supported_language_pairs = $this->getLanguagePairs($translator);
      // Build a mapping of source and target language pairs.
      foreach ($supported_language_pairs as $language) {
        $language_pairs[] = [
          'source_language' => $language['source']['code'],
          'target_language' => $language['target']['code'],
        ];
      }
    }
    catch (\Exception $e) {
      return [];
    }

    return $language_pairs;
  }

  /**
   * Gets the available language pairs.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $source_language
   *   (optional) The source language.
   *
   * @return array
   *   A list of language pairs.
   */
  public function getLanguagePairs(TMGMTTranslator $translator, $source_language = '') {
    $options = [];
    if (!empty($source_language)) {
      $options['sourcelang'] = $source_language;
    }

    try {
      return $this->doRequest($translator, 'GetLanguagePairs', $options);
    }
    catch (TMGMTException $e) {
      return [];
    }
  }

  /**
   * Gets the account info.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   *
   * @return array
   *   A list of account information.
   */
  public function getAccount(TMGMTTranslator $translator) {
    try {
      return $this->doRequest($translator, 'GetAccount');
    }
    catch (TMGMTException $e) {
      return [];
    }
  }

  /**
   * Creates an order.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $name
   *   The order name.
   * @param string|null $comment
   *   (optional) The comment for Acclaro.
   * @param string|null $duedate
   *   (optional) The due date for Acclaro.
   *
   * @return array
   *   A list of created order information.
   *
   * @throws \TMGMTException
   */
  public function createOrder(TMGMTTranslator $translator, $name, $comment = NULL, $duedate = NULL) {
    $query = [
      'name' => $name,
      'comments' => $comment,
      'duedate' => $duedate ? date_iso8601(strtotime($duedate['year'] . '-' . $duedate['month'] . '-' . $duedate['day'])) : NULL,
    ];
    return $this->doRequest($translator, 'CreateOrder', $query);
  }

  /**
   * Gets the order data.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   *
   * @return array
   *   A list of order information.
   *
   * @throws \TMGMTException
   */
  public function getOrder(TMGMTTranslator $translator, $order_id) {
    return $this->doRequest($translator, 'GetOrder', ['orderid' => $order_id]);
  }

  /**
   * Sets an order into complete state via simulate call.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   *
   * @return array
   *   A list of order information.
   *
   * @throws \TMGMTException
   */
  public function simulateOrderComplete(TMGMTTranslator $translator, $order_id) {
    return $this->doRequest($translator, 'SimulateOrderComplete', ['orderid' => $order_id]);
  }

  /**
   * Adds a comment to the file.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   * @param string $file_id
   *   The file ID.
   * @param string $comment
   *   The file comment.
   *
   * @return array
   *   A list of file information.
   *
   * @throws \TMGMTException
   */
  public function addFileComment(TMGMTTranslator $translator, $order_id, $file_id, $comment) {
    return $this->doRequest($translator, 'AddFileComment', [
      'orderid' => $order_id,
      'fileid' => $file_id,
      'comment' => $comment,
    ]);
  }

  /**
   * Adds a comment to the order.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   * @param string $comment
   *   The order comment.
   *
   * @return array
   *   A list of order information.
   *
   * @throws \TMGMTException
   */
  public function addOrderComment(TMGMTTranslator $translator, $order_id, $comment) {
    return $this->doRequest($translator, 'AddOrderComment', [
      'orderid' => $order_id,
      'comment' => $comment,
    ]);
  }

  /**
   * Adds a content disposition element.
   *
   * @param string $name
   *   Element name.
   * @param string $value
   *   Element value.
   * @param string $boundary
   *   Boundary value.
   *
   * @return string
   *   A content disposition element.
   */
  protected function addContentDispositionElement($name, $value, $boundary) {
    return "--$boundary\r\nContent-Disposition: form-data; name=\"$name\"\r\n\r\n$value\r\n";
  }

  /**
   * Uploads a file, and attaches it to the given order.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $source
   *   The source language.
   * @param string $target
   *   The target language.
   * @param string $order_id
   *   The order ID.
   * @param string $job_item_id
   *   The job item ID.
   * @param string $file_name
   *   The file name.
   * @param string $xliff_content
   *   The xliff content.
   *
   * @return array
   *   A list of information about submitted source file.
   *
   * @throws \TMGMTException
   */
  public function sendSourceFile(TMGMTTranslator $translator, $source, $target, $order_id, $job_item_id, $file_name, $xliff_content) {
    $boundary = '------------------------iBWEmnE4qy';
    $headers = ['Content-Type' => "multipart/form-data; boundary=$boundary"];

    $output = "";
    $output .= "--$boundary\r\n";
    $output .= "Content-Disposition: form-data; name=\"file\"; filename=\"$file_name\"\r\nContent-Type: text/plain\r\n\r\n";
    $output .= $xliff_content;
    $output .= "\r\n\r\n";
    $output .= $this->addContentDispositionElement('orderid', $order_id, $boundary);
    $output .= $this->addContentDispositionElement('sourcelang', $source, $boundary);
    $output .= $this->addContentDispositionElement('targetlang', $target, $boundary);
    $output .= $this->addContentDispositionElement('clientref', $job_item_id, $boundary);
    $output .= "--$boundary--";

    return $this->doRequest($translator, 'SendSourceFile', $output, 'POST', $headers);
  }

  /**
   * Submits the order.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   *
   * @return array
   *   A list of information about the order.
   *
   * @throws \TMGMTException
   */
  public function submitOrder(TMGMTTranslator $translator, $order_id) {
    return $this->doRequest($translator, 'SubmitOrder', ['orderid' => $order_id]);
  }

  /**
   * Gets the file info.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   *
   * @return array
   *   A list of information about the order files.
   *
   * @throws \TMGMTException
   */
  public function getFileInfo(TMGMTTranslator $translator, $order_id) {
    return $this->doRequest($translator, 'GetFileInfo', ['orderid' => $order_id]);
  }

  /**
   * Request that Acclaro invoke the supplied URL when status changes.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   * @param string $job_id
   *   The job ID.
   *
   * @return array
   *   A list of information about the order.
   *
   * @throws \TMGMTException
   */
  public function requestOrderCallback(TMGMTTranslator $translator, $order_id, $job_id) {
    $url = url("tmgmt_acclaro/callback/order/$job_id/$order_id", ['absolute' => TRUE]);
    return $this->doRequest($translator, 'RequestOrderCallback', [
      'orderid' => $order_id,
      'url' => $url,
    ]);
  }

  /**
   * Request that Acclaro no longer invoke the supplied URL when order changes.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   * @param string $job_id
   *   The job ID.
   *
   * @return array
   *   A list of information about the canceled order.
   *
   * @throws \TMGMTException
   */
  public function cancelOrderCallback(TMGMTTranslator $translator, $order_id, $job_id) {
    $url = url("tmgmt_acclaro/callback/order/$job_id/$order_id", ['absolute' => TRUE]);
    return $this->doRequest($translator, 'CancelOrderCallback', [
      'orderid' => $order_id,
      'url' => $url,
    ]);
  }

  /**
   * Request that Acclaro invoke the supplied URL when file changes.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   * @param string $file_id
   *   The file ID.
   * @param string $job_item_id
   *   The job item ID.
   *
   * @return array
   *   A list of information about the file.
   *
   * @throws \TMGMTException
   */
  public function requestFileCallback(TMGMTTranslator $translator, $order_id, $file_id, $job_item_id) {
    $url = url("tmgmt_acclaro/callback/file/$job_item_id/$order_id/$file_id", ['absolute' => TRUE]);
    return $this->doRequest($translator, 'RequestFileCallback', [
      'orderid' => $order_id,
      'fileid' => $file_id,
      'url' => $url,
    ]);
  }

  /**
   * Request that Acclaro no longer invoke the supplied URL when file changes.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   * @param string $file_id
   *   The file ID.
   * @param string $job_item_id
   *   The job item ID.
   *
   * @return array
   *   A list of information about the canceled file.
   *
   * @throws \TMGMTException
   */
  public function cancelFileCallback(TMGMTTranslator $translator, $order_id, $file_id, $job_item_id) {
    $url = url("tmgmt_acclaro/callback/file/$job_item_id/$order_id/$file_id", ['absolute' => TRUE]);
    return $this->doRequest($translator, 'CancelFileCallback', [
      'orderid' => $order_id,
      'fileid' => $file_id,
      'url' => $url,
    ]);
  }

  /**
   * Retrieves the translated file.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   * @param string $file_id
   *   The file ID.
   *
   * @return array|string
   *   A file content.
   *
   * @throws \TMGMTException
   */
  public function getFile(TMGMTTranslator $translator, $order_id, $file_id) {
    return $this->doRequest($translator, 'GetFile', [
      'orderid' => $order_id,
      'fileid' => $file_id,
    ]);
  }

  /**
   * Gets the file status.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param string $order_id
   *   The order ID.
   * @param string $file_id
   *   The file ID.
   *
   * @return array
   *   A list of information about the file status.
   *
   * @throws \TMGMTException
   */
  public function getFileStatus(TMGMTTranslator $translator, $order_id, $file_id) {
    return $this->doRequest($translator, 'GetFileStatus', [
      'orderid' => $order_id,
      'fileid' => $file_id,
    ]);
  }

  /**
   * Updates translation for the given source file, order ID.
   *
   * @param TMGMTJobItem $job_item
   *   The job item.
   * @param string $order_id
   *   The order ID.
   * @param string $file_id
   *   The file ID.
   * @param bool $add_message
   *   (optional) Add status message flag. Defaults to TRUE.
   *
   * @return bool
   *   Returns if the translation is fetched. Otherwise, FALSE.
   *
   * @throws \Exception|TMGMTException
   *   Throws an exception in case of invalid translation.
   */
  public function updateTranslation(TMGMTJobItem $job_item, $order_id, $file_id, $add_message = TRUE) {
    $translator = $job_item->getTranslator();
    /** @var \TMGMTAcclaroTranslatorPluginController $controller */
    $controller = $job_item->getTranslatorController();
    // Get the file status info.
    $file_status = $controller->getFileStatus($translator, $order_id, $file_id);

    switch ($file_status['status']) {
      case 'complete':
        // If the status is "complete", remote translation is ready.
        $controller->importTranslation($translator, $job_item, $order_id, $file_status['targetfile']);
        return TRUE;

      case 'canceled':
        // Abort the item if the file is cancelled.
        $this->abortJobItem($translator, $job_item, $order_id, $file_id);
        return FALSE;

      default:
        if ($add_message) {
          $job_item->addMessage('The remote translation has changed the status to %status.', ['%status' => $file_status['status']]);
        }
        return FALSE;
    }
  }

  /**
   * Fetches translations for job items of a given job.
   *
   * @param TMGMTJob $job
   *   A job containing job items that translations will be fetched for.
   */
  public function fetchTranslations(TMGMTJob $job) {
    /** @var \TMGMTRemote[] $remotes */
    $remotes = $job->getRemoteMappings();
    $translated = 0;

    // Check for if there are completed translations.
    foreach ($remotes as $remote) {
      $job_item = $remote->getJobItem();
      $file_id = $remote->remote_identifier_1;
      $order_id = $remote->remote_identifier_2;

      try {
        if ($status = $this->updateTranslation($job_item, $order_id, $file_id, FALSE)) {
          $translated++;
        }
      }
      catch (TMGMTException $tmgmt_exception) {
        $job_item->addMessage($tmgmt_exception->getMessage());
      }
      catch (\Exception $e) {
        watchdog_exception('tmgmt_acclaro', $e);
      }
    }

    // Provide a message about translated items.
    if ($translated == 0) {
      drupal_set_message(t('No job item has been translated yet.'), 'warning');
    }
    else {
      $untranslated = count($remotes) - $translated;
      if ($untranslated > 0) {
        $job->addMessage('Fetched translations for @translated job items, @untranslated items are not translated yet.', [
          '@translated' => $translated,
          '@untranslated' => $untranslated,
        ]);
      }
      else {
        $job->addMessage('Fetched translations for @translated job items.', ['@translated' => $translated]);
      }
      tmgmt_ui_write_request_messages($job);
    }
  }

  /**
   * Simulates complete orders from Acclaro.
   *
   * @param TMGMTJob $job
   *   The translation job.
   */
  public function simulateCompleteOrder(TMGMTJob $job) {
    $remote_mappings = $job->getRemoteMappings();
    $remote_mapping = reset($remote_mappings);

    if ($remote_mapping) {
      $order_id = $remote_mapping->remote_identifier_2;

      try {
        $order = $this->simulateOrderComplete($job->getTranslator(), $order_id);
        if ($order['status'] == 'complete') {
          $job->addMessage(t('The order (@order_id) has been marked as completed by using simulate order complete command.', ['@order_id' => $order_id]));
        }
      }
      catch (TMGMTException $e) {
        drupal_set_message($e->getMessage(), 'error');
      }
    }
  }

  /**
   * Retrieves the translation.
   *
   * @param \TMGMTTranslator $translator
   *   The translator.
   * @param \TMGMTJobItem $job_item
   *   The job item.
   * @param string $order_id
   *   The order ID.
   * @param string $target_file_id
   *   The target file ID.
   *
   * @throws \TMGMTException
   */
  public function importTranslation(TMGMTTranslator $translator, TMGMTJobItem $job_item, $order_id, $target_file_id) {
    // Check the target file status.
    $target_file_status = $this->getFileStatus($translator, $order_id, $target_file_id);
    // Abort the import in case its status is not "complete".
    if ($target_file_status['status'] != 'complete') {
      throw new TMGMTException('The remote translation is not completed yet.');
    }

    $xliff = tmgmt_file_format_controller('xlf');
    // Fetch and validate the remote translation.
    $translation = $this->getFile($translator, $order_id, $target_file_id);
    $validated_job = $xliff->validateImport($translation, FALSE);
    if (!$validated_job) {
      throw new TMGMTException('Failed to validate remote translation, import aborted.');
    }
    elseif ($validated_job->tjid != $job_item->getJob()->tjid) {
      throw new TMGMTException('The remote translation (File ID: @file_id, Job ID: @target_job_id) does not match the current job ID @job_id.', [
        '@file_id' => $target_file_id,
        '@target_job_job' => $validated_job->tjid,
        '@job_id' => $job_item->getJob()->tjid,
      ], 'error');
    }
    else {
      if ($data = $xliff->import($translation, FALSE)) {
        // The remote translation was successfully imported.
        $job_item->getJob()->addTranslatedData($data, NULL, TMGMT_DATA_ITEM_STATE_TRANSLATED);
        $job_item->addMessage('The translation has been received.');
      }
      else {
        throw new TMGMTException('Could not process received translation data for the target file @file_id.', ['@file_id' => $target_file_id]);
      }
    }
  }

  /**
   * Aborts a job item and send a comment to Acclaro.
   *
   * @param TMGMTTranslator $translator
   *   The translator.
   * @param TMGMTJobItem $job_item
   *   The job item to abort.
   * @param string $order_id
   *   The order ID.
   * @param string $file_id
   *   The file ID.
   */
  public function abortJobItem(TMGMTTranslator $translator, TMGMTJobItem $job_item, $order_id, $file_id) {
    if (!$job_item->isAborted()) {
      try {
        $this->addFileComment($translator, $order_id, $file_id, 'CANCEL FILE');
        // Abort the job item.
        $uri = $job_item->getSourceUri();
        $message = 'The translation of !source has been aborted by the user.';
        $variables = ['!source' => l($job_item->getSourceLabel(), $uri['path'])];
        $job_item->setState(TMGMT_JOB_ITEM_STATE_ABORTED, $message, $variables);
        // As file was aborted, tell Acclaro that no more updates are needed.
        $this->cancelFileCallback($translator, $order_id, $file_id, $job_item->tjiid);
      }
      catch (TMGMTException $e) {
        $uri = $job_item->getSourceUri();
        $message = 'Failed to abort !source item. @error';
        $variables = [
          '!source' => l($job_item->getSourceLabel(), $uri['path']),
          '@error' => $e->getMessage(),
        ];
        $job_item->addMessage($message, $variables, 'error');
      }
    }
  }

  /**
   * Aborts normal jobs.
   *
   * @param TMGMTJob $job
   *   The job to abort.
   *
   * @return bool
   *   Returns TRUE if job was aborted. Otherwise, FALSE.
   */
  public function abortJob(TMGMTJob $job) {
    $order_id = NULL;
    $translator = $job->getTranslator();
    /** @var \TMGMTRemote $remote */
    foreach ($job->getRemoteMappings() as $remote) {
      $job_item = $remote->getJobItem();
      $file_id = $remote->remote_identifier_1;
      $order_id = $remote->remote_identifier_2;

      // Abort the job item and notify Acclaro via file comment.
      $this->abortJobItem($translator, $job_item, $order_id, $file_id);
    }

    try {
      // Abort job in the current system.
      $aborted = FALSE;
      if ($job->isAbortable()) {
        $job->setState(TMGMT_JOB_STATE_ABORTED, 'Translation job has been aborted.');
        $aborted = TRUE;
      }
      // Notify Acclaro about aborted order.
      if ($order_id && $aborted) {
        $this->addOrderComment($translator, $order_id, 'CANCEL ORDER');
        // As order was aborted, tell Acclaro that no more updates are needed.
        $this->cancelOrderCallback($translator, $order_id, $job->tjid);
        return TRUE;
      }
    }
    catch (TMGMTException $e) {
      $job->addMessage('Failed to abort translation job. @error', ['@error' => $e->getMessage()], 'error');
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function abortTranslation(TMGMTJob $job) {
    drupal_set_message(t('Please contact your Acclaro project manager with the order cancellation request.'), 'warning');
    return $this->abortJob($job);
  }

}
